﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using SmarxToDo_Web.Models;
using System.Xml.Serialization;
using System.IO;
using System.Data.Services.Client;
using Microsoft.WindowsAzure.ServiceRuntime;

namespace SmarxToDo_Web.Controllers
{
    public class ApiController : Controller
    {
        [HttpPost]
        public ActionResult Add(string task)
        {
            var account = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString"));
            var ctx = account.CreateCloudTableClient().GetDataServiceContext();

            ctx.AddObject("Tasks", new TaskEntity(task));
            ctx.SaveChangesWithRetries();

            return new EmptyResult();
        }

        [HttpPost]
        [ValidateInput(false)]
        public ActionResult Get(string token)
        {
            var account = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString"));
            var ctx = account.CreateCloudTableClient().GetDataServiceContext();
            var query = ctx.CreateQuery<TaskEntity>("Tasks")
                .Where(t => t.PartitionKey == string.Empty).Take(5).AsTableServiceQuery();
            var continuation = token != null
                               ? (ResultContinuation)new XmlSerializer(typeof(ResultContinuation)).Deserialize(new StringReader(token))
                               : null;
            var response = query.EndExecuteSegmented(query.BeginExecuteSegmented(continuation, null, null));
            var sw = new StringWriter();
            new XmlSerializer(typeof(ResultContinuation)).Serialize(sw, response.ContinuationToken);
            return Json(new {
                tasks = response.Results,
                nextToken = sw.ToString(),
                hasMore = response.ContinuationToken != null });
        }

        [HttpPost]
        public ActionResult Delete(string rowKey)
        {
            var account = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString"));
            var ctx = account.CreateCloudTableClient().GetDataServiceContext();

            var task = new TaskEntity { PartitionKey = string.Empty, RowKey = rowKey };
            ctx.AttachTo("Tasks", task, "*");
            ctx.DeleteObject(task);
            ctx.SaveChangesWithRetries();

            return new EmptyResult();
        }

        [HttpPost]
        public ActionResult Complete(string rowKey, bool completed)
        {
            var account = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString"));
            var ctx = account.CreateCloudTableClient().GetDataServiceContext();

            var task = ctx.CreateQuery<TaskEntity>("Tasks")
                .Where(t => t.PartitionKey == string.Empty && t.RowKey == rowKey).Single();
            task.Completed = completed;
            ctx.UpdateObject(task);
            ctx.SaveChangesWithRetries();

            return new EmptyResult();
        }
    }
}